(ns test.core
  (:require [quil.core :as q]
            [quil.middleware :as m]))

(defn setup []
  ; Set frame rate to 30 frames per second.
  (q/frame-rate 30)
  ; Set color mode to HSB (HSV) instead of default RGB.
  (q/color-mode :hsb)
  ; setup function returns initial state. It contains
  ; circle color and position.
  {:color 0
   :angle 0
   :pos 0})

(defn update-state [state]
  ; Update sketch state by changing circle color and position.
  {:color (mod (+ (:color state) 0.7) 255)
   :angle (+ (:angle state) 0.1)
   :pos (mod (+ (:pos state) 1) 140)})

(defn draw-state [state]
  ; Clear the sketch by filling it with light-grey color.
  (q/background 240)
  ; Set circle color.
  (q/fill (:color state) 255 255)
    (q/with-translation [(/ (q/width) 2)
                         (/ (q/height) 2)]
      (doseq [i (range 10 20 2)]
        (let [angle (:angle state)
              pos (:pos state)
              x (* (* i 17) (q/cos angle))
              y (* (* i 12) (q/sin angle))
              size 30]
          (q/ellipse x y size size)
          (q/ellipse pos x size size)
          (q/ellipse y x size size)
      )))) ; #'test.core/draw-state


(q/defsketch test
  :title "You spin my circle right round"
  :size [500 500]
  ; setup function called only once, during sketch initialization.
  :setup setup
  ; update-state is called on each iteration before draw-state.
  :update update-state
  :draw draw-state
  :features [:keep-on-top]
  ; This sketch uses functional-mode middleware.
  ; Check quil wiki for more info about middlewares and particularly
  ; fun-mode.
  :middleware [m/fun-mode])
